@@ -74,56 +74,65 @@ def apply_pnl(row):
 
     open_date = row.open_date
     close_date = row.date
-    now_close = df_daily.at[open_date, 'close']
+    now_close = float(df_daily.at[open_date, 'close'])
     dates = pd.date_range(start=open_date, end=close_date)
-    open_price = row.open_price
-    num_of_share = row.num_of_share
-    commission = row.commission
+    open_price = float(row.open_price)
+    num_of_share = int(row.num_of_share)
+    commission = float(row.commission)
     action = row.action
-    realized_pnl = row.realized_pnl
+    realized_pnl = float(row.realized_pnl)
 
     last_realized_capital = apply_pnl.last_realized_capital
-    multiplier = apply_pnl.multiplier
+    multiplier = float(apply_pnl.multiplier)
+
 
     for date in dates:
-        if date == open_date:
-            # Unrealized pnl on the open date, record on df_daily
-            df_daily.at[open_date, 'action'] = 'open'
-            unrealized_pnl = num_of_share * multiplier * (now_close - open_price) - commission
-            unrealized_pnl = round(unrealized_pnl, 3)
-            df_daily.at[open_date, 'unrealized_pnl'] = unrealized_pnl
+        if (date == open_date or date == close_date):
+            # open_date and close_date may be the first
+            # therefore, process open_date first, then close_date
 
-            # print(open_date, 'Open!', open_price, now_close, num_of_share, unrealized_pnl)
+            if date == open_date:
+                # Unrealized pnl on the open date, record on df_daily
+                df_daily.at[open_date, 'action'] = 'open'
 
-        elif date == close_date:  # close position on this date, end of this trade
-            # this is a same day trade
-            # clear the open unrealized pnl as open and close same day should not aggregate
-            # it should be record as realized pnl directly
-            if open_date == close_date:
-                df_daily.at[date, 'unrealized_pnl'] = None
+                unrealized_pnl = num_of_share * multiplier * (now_close - open_price)#  - commission
+                unrealized_pnl = round(unrealized_pnl, 3)
+                df_daily.at[open_date, 'unrealized_pnl'] = unrealized_pnl
 
-            # unrealized_pnl
-            if df_daily.at[date, 'unrealized_pnl']:  # if there is already unrealized_pnl, aggregate
-                df_daily.at[date, 'unrealized_pnl'] = df_daily.at[date, 'unrealized_pnl'] + realized_pnl
-            else:
-                df_daily.at[date, 'unrealized_pnl'] = realized_pnl
+                # print(open_date, 'Open!', open_price, now_close, num_of_share, unrealized_pnl)
 
-            # realized pnl
-            if df_daily.at[date, 'realized_pnl']:  # if there is already realized_pnl, aggregate
-                df_daily.at[date, 'realized_pnl'] = df_daily.at[date, 'realized_pnl'] + realized_pnl
-            else:
-                df_daily.at[date, 'realized_pnl'] = realized_pnl
+            if date == close_date:  # close position on this date, end of this trade
+                # this is a same day trade
+                # clear the open unrealized pnl as open and close same day should not aggregate
+                # it should be record as realized pnl directly
+                if open_date == close_date:
+                    df_daily.at[date, 'unrealized_pnl'] = None
 
-            df_daily.at[date, 'action'] = action
-            df_daily.at[date, 'commission'] = commission
-            last_realized_capital = last_realized_capital + realized_pnl
+                # unrealized_pnl
+                if df_daily.at[date, 'unrealized_pnl']:  # if there is already unrealized_pnl, aggregate
+                    df_daily.at[date, 'unrealized_pnl'] = df_daily.at[date, 'unrealized_pnl'] + realized_pnl
+                else:
+                    df_daily.at[date, 'unrealized_pnl'] = realized_pnl
 
-            # print(date, 'Close!', open_price, close_date, realized_pnl)
-            # print()
+                # realized pnl
+                if df_daily.at[date, 'realized_pnl']:  # if there is already realized_pnl, aggregate
+                    df_daily.at[date, 'realized_pnl'] = df_daily.at[date, 'realized_pnl'] + realized_pnl
+                else:
+                    df_daily.at[date, 'realized_pnl'] = realized_pnl
+
+                df_daily.at[date, 'action'] = action
+                df_daily.at[date, 'commission'] = commission
+                last_realized_capital = last_realized_capital + realized_pnl
+
+                # print(date, 'Close!', open_price, close_date, realized_pnl)
+                # print()
 
         else:
+            # it is not (date == open_date or date == close_date), so it is between two date
+            # calculate unrealized pnl only
             try:
                 now_close = df_daily.at[date, 'close']
+
                 unrealized_pnl = num_of_share * multiplier * (now_close - open_price) - commission
                 unrealized_pnl = round(unrealized_pnl, 3)
                 if df_daily.at[date, 'unrealized_pnl']:
@@ -385,7 +394,7 @@ def resample_summary_to_daily(para_combination, folder=''):
     # Calculate initial capital before trim the dataframe
     if len(df) == 0:
         # This would happen when it is summary mode and no trade for the strategy
-        # Note that in intra day but not summary mode, length of df of  no trade df_bakctest is still > 0
+        # Note that in intraday but not summary mode, length of df of  no trade df_bakctest is still > 0
         initial_capital = sec_profile['initial_capital']
     else:
         if pl.sum(df.get_column('realized_pnl')) == None:  # when NOT summary mode but no trade
@@ -440,6 +449,7 @@ def resample_summary_to_daily(para_combination, folder=''):
         pl.lit(None).alias('action'),
         pl.lit(None).alias('realized_pnl'),
         pl.lit(None).alias('unrealized_pnl'),
+        pl.lit(None).alias('commission'),
     ])
           .collect()
           )
@@ -451,6 +461,7 @@ def resample_summary_to_daily(para_combination, folder=''):
     apply_pnl.last_realized_capital = last_realized_capital
     apply_pnl.multiplier = multiplier
     apply_pnl.df_daily = df_daily
+
     df_backtest.apply(apply_pnl, axis=1)
 
     df_daily = apply_pnl.df_daily
@@ -489,6 +500,7 @@ def cal_performance(para_combination):
 
     df = df_daily
 
+
     # Deter if equity_value unchange = no tade
     equity_value_column = df['equity_value'].to_numpy()
     no_trade = (equity_value_column[0] == equity_value_column).all()
@@ -570,7 +582,8 @@ def cal_performance(para_combination):
             result_dict['net_profit_to_mdd'] = net_profit / mdd_dollar
 
         # Cov
-        df3 = df[df['action'] == 'open']
+        # df3 = df[df['action'] == 'open']
+        df3 = df[df['action'].notnull()]
         df3 = df3.set_index('date')
         signal_year_count = df3.groupby(lambda x: x.year).size()
 
